#


=head1 NAME

Robert's Web Site Monitor v.4	3-4-03

=head1 SYNOPSIS

Sends an alert message to any email-able device when a specified web site goes down, logs response time of site

=head1 AUTHOR

Robert Bleidt

=head1 SEE ALSO

from "http://www.hdtv.com/rb/" pages

=head1 COPYRIGHT

Copyright (c) 2003 Robert L. Bleidt. ALL RIGHTS RESERVED.
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 

=cut

$VERSION = "4.00";

# ------------------  local configuration stuff

# stuff you have to fill in:

$nameserver = "192.168.1.1";					# nameserver reachable by this machine
	# this should be one of the DNS server entries for your machine
	# on a win2000 PC, run "ipconfig /all" from a command prompt to find out

$MyEmailDomain = 'nobody.com';					# this is the domain name to use when sending email (you might put your web site's domain here) 
$Website = 'http://www.nobody.com/';			# this is web page to monitor
$WebsiteCheckPhrase = 'Some text at the bottom of the page';  # this is the text string to look for in the page

# you also must fill in the lines below the comments marked with "===" below

# stuff you can change if you want to

$logFileName = ">>C:\\webMonitorLog.txt"; 		# file where status and errors go
$statsFileName = ">>C:\\webMonitorStats.txt";	# file where response times go



# uses: Net::DNS, Net::SMTP, Time::HiRes, LWP::UserAgent  

#-------------------  routines to write out log and statistics files


sub writeLogFile ($) {

  my ($msg) = @_;
  my $timeText = localtime;
  open(LOGFILE,$logFileName);
  print LOGFILE "$timeText $msg \n";
  close(LOGFILE);
  print "$timeText $msg \n";
}

{
my $oldDay = -1;
my @dailyStats;


sub writeStats ($) {
  my ($stat) = @_;
  my $timeText = localtime;
  my $avg = 0,$count = 0;

  open(STATSFILE,$statsFileName);

  ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime;
  if ($wday != $oldDay) { # start each day on a new line
    $oldDay = $wday;
    foreach $num (@dailyStats) {
      $avg += $num;
      $count++;
    }
    if ($count > 0) {
      $avg /= $count;
      printf STATSFILE "(%s)", $avg;
    }
    @dailyStats = ();
    printf STATSFILE "\n[%s] ", $timeText;
  }

  if ($stat > 0) {push @dailyStats,$stat;}

  printf STATSFILE "%.2f ",$stat;
  close(STATSFILE);
}
}

#-------------------  get MX records for a domain
sub getMX ($) {

  use Net::DNS;

  my ($dname) = @_;

  $res = new Net::DNS::Resolver;

  $res->nameservers($nameserver); 
  my @mx = mx($res, $dname);
  if (@mx) {
  #    foreach $rr (@mx) {	
  #        print $rr->preference, " ", $rr->exchange, "\n"; 
  #    }
  }
  else {
      writeLogFile ("can't find MX records for $dname:" . $res->errorstring . "\n");
  }

  @mx = sort {$a->preference <=> $b->preference} @mx;
  return (@mx);
}

#-------------------  send an email message
sub mailMsg ($$$$) {

  my ($from, $to, $subject, $message) = @_;
  my ($toUser,$toDomain) = split /@/,$to;

  use Net::SMTP;

  @mxrecs = getMX($toDomain);

  foreach $mxrec (@mxrecs) 
  {

    my $errors = 0;
    $timeText = localtime;
    open(LOGFILE,$logFileName);
    print LOGFILE "$timeText mailMsg user = $toUser domain = $toDomain exchanger = ", $mxrec->exchange,"\n";

    print LOGFILE "$timeText Sending error message to ";

    if ( $smtp = Net::SMTP->new($mxrec->exchange,Hello=>$MyEmailDomain,Timeout=>60,Debug =>1)  ) 
    {

      print 'Domain: ',$smtp->domain,"\n";
      print LOGFILE $smtp->domain," ";

      if ($smtp->mail($from)) 
      {
        if ($smtp->to($to)) 
        {
          unless ($smtp->data("From: $from\n" , "To: $to \n" , "Subject: $subject \n\n" , $message)) 
            {print LOGFILE "DATA Failed "; $errors++;}
        }
        else {print LOGFILE "TO Failed "; $errors++;}
      }
      else {print LOGFILE "MAIL Failed "; $errors++;}
      unless ($smtp->quit) {print LOGFILE "QUIT Failed ";$errors++;}
    }
    else {print LOGFILE "NEW Failed "; $errors++;}

    print LOGFILE "\n"; 
    close(LOGFILE);
    unless ($errors) {return(1);}
  }
  return;
}

#-------------------  when site is down, this routine sends mail to addresses specified
{
my $lastMsg = 0;

sub mailErrorMsg () {

  if (time() > $lastMsg + 21600) #21600 = 6 hours - only send a message this often - to keep from enraging user
  {

#=== the other stuff to fill in:
#===
#=== here you should insert a copy of the mailMsg line for each person or device you want emailed when the site goes down 
#=== the arguments are:
#=== 1 - your email address (this will be in the From field of the email)
#=== 2 - the address to send the email to 
#=== 3 - the subject line of the message
#=== 4 - the body of the message (optional)
#=== 
#=== for example, if my website is www.nobody.com and my AT&T mobile phone is 800-555-1234
#=== I would use the following line to send a text message to my phone:
#===
#===     mailMsg('webmaster@nobody.com','8005551234@mobile.att.net','Nobody.com Web Server Down','');
#===
#=== usually, your phone or pager company's web pages will have instructions on how to send a message using email
#=== 
#=== another example: if my day job is at bigco.com, I could have it send me an email there:
#===
#===     mailMsg('webmaster@nobody.com','me@bigco.com','Nobody.com Web Server Down','');
#===
	mailMsg('webmaster@nobody.com','8005551234@mobile.att.net','Nobody.com Web Server Down','');
	mailMsg('webmaster@nobody.com','me@bigco.com','Nobody.com Web Server Down','');

    $lastMsg = time();
  }
}
}


#-------------------  check web site page for a check phrase
sub checkWebsite ($$) {
  my ($url,$checkPhrase) = @_;
  # returns response time of site in seconds, -1 if not responding

  use Time::HiRes qw ( gettimeofday tv_interval );
  use LWP::UserAgent;

  my $ua = LWP::UserAgent->new;  # Create a user agent object

  $ua->agent("Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0) ");
  $ua->timeout(30);

  # send request
  my $req = HTTP::Request->new(GET => $url);
  $req->content_type('text/html');

  my $time0,$time1,$resTime;
  $time0 = [gettimeofday];

  my $res = $ua->request($req);

  $time1 = [gettimeofday];
  $resTime = tv_interval ($time0, $time1);

#print "$url: response time = $resTime\n"; 
    

  # Check response
  if ($res->is_success && $res->content =~ /$checkPhrase/ && $res->content =~ m'</html>') {
    return ($resTime);
  } 
  else {
    return (-1);
  }
}


#----------------------------  main routine

  my $resTime; # time for website to respond with page
  my $pollInterval = 0;      # time between website checks 
  my $normalPoll = 3600;   # 900 = 15 min, 3600 = 1 hr, 180 = 3 min
  my $fastPoll = 180;

  print "Robert's Website Monitor $VERSION\nCopyright (c) 2003 Robert L. Bleidt ALL RIGHTS RESERVED.\nSee code for details.\n\n";
  writeLogFile ("Starting Website Monitor");
  
  while (1) {
    $resTime = checkWebsite($Website,$WebsiteCheckPhrase);
    if($resTime > 0) {
      # website good, log response time
      writeStats($resTime);
      if ($pollInterval <= $fastPoll) { writeLogFile ("Website Up\n"); }
      $pollInterval = $normalPoll;
    }
    else {
      $pollInterval = $fastPoll;
      $resTime = checkWebsite('http://www.google.com/','google');
      if ($resTime > 0) {
        writeStats(-1.0); # we can connect to google, our site is down
        writeLogFile ("Website Down\n");
        mailErrorMsg ();
      }
      else {
        $resTime = checkWebsite('http://www.yahoo.com/','yahoo');
        if ($resTime > 0) {
          writeStats(-1.0); # we can't get google, but yahoo is OK, our site must be down
          writeLogFile ("Website Down\n");
          mailErrorMsg ();
        }
        else {
          writeStats(-2.0); # we can't get google or yahoo, local network must be down
          writeLogFile ("Network Down\n");
       }
      }
    }
    sleep($pollInterval);     
  }

__END__